/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019, Mellanox Technologies. All rights reserved.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
#ifndef __L3CACHE_REGS_H__
#define __L3CACHE_REGS_H__

#ifdef __ASSEMBLER__
#define _64bit(x) x
#else /* __ASSEMBLER__ */
#define _64bit(x) x ## ULL
#endif /* __ASSEMBLER */

#include <linux/types.h>

#define L3C_PROF_RD_MISS__FIRST_WORD 0x0600
#define L3C_PROF_RD_MISS__LAST_WORD 0x063c
#define L3C_PROF_RD_MISS__LENGTH 0x0040
#define L3C_PROF_RD_MISS__STRIDE 0x0004

#define L3C_PROF_RD_MISS__LOW_ORDER_SHIFT 0
#define L3C_PROF_RD_MISS__LOW_ORDER_WIDTH 5
#define L3C_PROF_RD_MISS__LOW_ORDER_RESET_VAL 11
#define L3C_PROF_RD_MISS__LOW_ORDER_RMASK 0x1f
#define L3C_PROF_RD_MISS__LOW_ORDER_MASK  0x1f

#define L3C_PROF_RD_MISS__HIGH_ORDER_SHIFT 5
#define L3C_PROF_RD_MISS__HIGH_ORDER_WIDTH 5
#define L3C_PROF_RD_MISS__HIGH_ORDER_RESET_VAL 0
#define L3C_PROF_RD_MISS__HIGH_ORDER_RMASK 0x1f
#define L3C_PROF_RD_MISS__HIGH_ORDER_MASK  0x3e0

#define L3C_PROF_RD_MISS__ALLOC_STATE_SHIFT 12
#define L3C_PROF_RD_MISS__ALLOC_STATE_WIDTH 1
#define L3C_PROF_RD_MISS__ALLOC_STATE_RESET_VAL 0
#define L3C_PROF_RD_MISS__ALLOC_STATE_RMASK 0x1
#define L3C_PROF_RD_MISS__ALLOC_STATE_MASK  0x1000

#define L3C_PROF_RD_MISS__LOW_STATE_BLK_ALLOC_SHIFT 13
#define L3C_PROF_RD_MISS__LOW_STATE_BLK_ALLOC_WIDTH 1
#define L3C_PROF_RD_MISS__LOW_STATE_BLK_ALLOC_RESET_VAL 0
#define L3C_PROF_RD_MISS__LOW_STATE_BLK_ALLOC_RMASK 0x1
#define L3C_PROF_RD_MISS__LOW_STATE_BLK_ALLOC_MASK  0x2000

#define L3C_PROF_RD_MISS__HIGH_STATE_BLK_ALLOC_SHIFT 14
#define L3C_PROF_RD_MISS__HIGH_STATE_BLK_ALLOC_WIDTH 1
#define L3C_PROF_RD_MISS__HIGH_STATE_BLK_ALLOC_RESET_VAL 0
#define L3C_PROF_RD_MISS__HIGH_STATE_BLK_ALLOC_RMASK 0x1
#define L3C_PROF_RD_MISS__HIGH_STATE_BLK_ALLOC_MASK  0x4000

#define L3C_PROF_RD_MISS__PROB_SHIFT 16
#define L3C_PROF_RD_MISS__PROB_WIDTH 16
#define L3C_PROF_RD_MISS__PROB_RESET_VAL 0
#define L3C_PROF_RD_MISS__PROB_RMASK 0xffff
#define L3C_PROF_RD_MISS__PROB_MASK  0xffff0000

#ifndef __ASSEMBLER__
__extension__
typedef union {
	struct {
		/*
		 * The LRU structure index to which read transactions should
		 * allocate, in case their corresponding binary randomization
		 * results in 0. Negative indexes (i.e. when MSB is 0) would
		 * mean that the transactions will not be allocated.
		 */
		u32 low_order            : 5;
		/*
		 * The LRU structure index to which read transactions should
		 * allocate, in case their corresponding binary randomization
		 * results in 1. Negative indexes (i.e. when MSB is 1) would
		 * mean that the transactions will not be allocated.
		 */
		u32 high_order           : 5;
		/* Reserved. */
		u32 __reserved_0         : 2;
		/*
		 * Specifies the initial state of the cache entry's State
		 * Bit, after a read transaction gets allocated into it.
		 */
		u32 alloc_state          : 1;
		/*
		 * Block allocation of read transaction, if the State Bit of
		 * the evicted cache entry is high, in case their
		 * corresponding binary randomization results in 0.
		 */
		u32 low_state_blk_alloc  : 1;
		/*
		 * Block allocation of read transaction, if the State Bit of
		 * the evicted cache entry is high, in case their
		 * corresponding binary randomization results in 1.
		 */
		u32 high_state_blk_alloc : 1;
		/* Reserved. */
		u32 __reserved_1         : 1;
		/*
		 * The probability of a read transaction that get a miss in
		 * the cache, to be allocated to the Read Miss High Order
		 * rather than to the Read Miss Low Order.
		 */
		u32 prob                 : 16;
	};

	u32 word;
} L3C_PROF_RD_MISS_t;
#endif /* !defined(__ASSEMBLER__) */

#define L3C_PROF_WR_MISS__FIRST_WORD 0x0640
#define L3C_PROF_WR_MISS__LAST_WORD 0x067c
#define L3C_PROF_WR_MISS__LENGTH 0x0040
#define L3C_PROF_WR_MISS__STRIDE 0x0004

#define L3C_PROF_WR_MISS__LOW_ORDER_SHIFT 0
#define L3C_PROF_WR_MISS__LOW_ORDER_WIDTH 5
#define L3C_PROF_WR_MISS__LOW_ORDER_RESET_VAL 11
#define L3C_PROF_WR_MISS__LOW_ORDER_RMASK 0x1f
#define L3C_PROF_WR_MISS__LOW_ORDER_MASK  0x1f

#define L3C_PROF_WR_MISS__HIGH_ORDER_SHIFT 5
#define L3C_PROF_WR_MISS__HIGH_ORDER_WIDTH 5
#define L3C_PROF_WR_MISS__HIGH_ORDER_RESET_VAL 0
#define L3C_PROF_WR_MISS__HIGH_ORDER_RMASK 0x1f
#define L3C_PROF_WR_MISS__HIGH_ORDER_MASK  0x3e0

#define L3C_PROF_WR_MISS__ALLOC_STATE_SHIFT 12
#define L3C_PROF_WR_MISS__ALLOC_STATE_WIDTH 1
#define L3C_PROF_WR_MISS__ALLOC_STATE_RESET_VAL 0
#define L3C_PROF_WR_MISS__ALLOC_STATE_RMASK 0x1
#define L3C_PROF_WR_MISS__ALLOC_STATE_MASK  0x1000

#define L3C_PROF_WR_MISS__LOW_STATE_BLK_ALLOC_SHIFT 13
#define L3C_PROF_WR_MISS__LOW_STATE_BLK_ALLOC_WIDTH 1
#define L3C_PROF_WR_MISS__LOW_STATE_BLK_ALLOC_RESET_VAL 0
#define L3C_PROF_WR_MISS__LOW_STATE_BLK_ALLOC_RMASK 0x1
#define L3C_PROF_WR_MISS__LOW_STATE_BLK_ALLOC_MASK  0x2000

#define L3C_PROF_WR_MISS__HIGH_STATE_BLK_ALLOC_SHIFT 14
#define L3C_PROF_WR_MISS__HIGH_STATE_BLK_ALLOC_WIDTH 1
#define L3C_PROF_WR_MISS__HIGH_STATE_BLK_ALLOC_RESET_VAL 0
#define L3C_PROF_WR_MISS__HIGH_STATE_BLK_ALLOC_RMASK 0x1
#define L3C_PROF_WR_MISS__HIGH_STATE_BLK_ALLOC_MASK  0x4000

#define L3C_PROF_WR_MISS__PROB_SHIFT 16
#define L3C_PROF_WR_MISS__PROB_WIDTH 16
#define L3C_PROF_WR_MISS__PROB_RESET_VAL 0
#define L3C_PROF_WR_MISS__PROB_RMASK 0xffff
#define L3C_PROF_WR_MISS__PROB_MASK  0xffff0000

#ifndef __ASSEMBLER__
__extension__
typedef union {
	struct {
		/*
		 * The LRU structure index to which write transactions should
		 * allocate, in case their corresponding binary randomization
		 * results in 0. Negative indexes (i.e. when MSB is 0) would
		 * mean that the transactions will not be allocated.
		 */
		u32 low_order            : 5;
		/*
		 * The LRU structure index to which write transactions should
		 * allocate, in case their corresponding binary randomization
		 * results in 1. Negative indexes (i.e. when MSB is 1) would
		 * mean that the transactions will not be allocated.
		 */
		u32 high_order           : 5;
		/* Reserved. */
		u32 __reserved_0         : 2;
		/*
		 * Specifies the initial state of the cache entry's State
		 * Bit, after a write transaction gets allocated into it.
		 */
		u32 alloc_state          : 1;
		/*
		 * Block allocation of write transaction, if the State Bit of
		 * the evicted cache entry is high, in case their
		 * corresponding binary randomization results in 0.
		 */
		u32 low_state_blk_alloc  : 1;
		/*
		 * Block allocation of write transaction, if the State Bit of
		 * the evicted cache entry is high, in case their
		 * corresponding binary randomization results in 1.
		 */
		u32 high_state_blk_alloc : 1;
		/* Reserved. */
		u32 __reserved_1         : 1;
		/*
		 * The probability of a write transaction that get a miss in
		 * the cache, to be allocated to the Write Miss High Order
		 * rather than to the Write Miss Low Order.
		 */
		u32 prob                 : 16;
	};

	u32 word;
} L3C_PROF_WR_MISS_t;
#endif /* !defined(__ASSEMBLER__) */

#define L3C_PROF_RD_HIT0__FIRST_WORD 0x0680
#define L3C_PROF_RD_HIT0__LAST_WORD 0x06bc
#define L3C_PROF_RD_HIT0__LENGTH 0x0040
#define L3C_PROF_RD_HIT0__STRIDE 0x0004

#define L3C_PROF_RD_HIT0__LOW_DELTA_SHIFT 0
#define L3C_PROF_RD_HIT0__LOW_DELTA_WIDTH 5
#define L3C_PROF_RD_HIT0__LOW_DELTA_RESET_VAL 11
#define L3C_PROF_RD_HIT0__LOW_DELTA_RMASK 0x1f
#define L3C_PROF_RD_HIT0__LOW_DELTA_MASK  0x1f

#define L3C_PROF_RD_HIT0__HIGH_DELTA_SHIFT 5
#define L3C_PROF_RD_HIT0__HIGH_DELTA_WIDTH 5
#define L3C_PROF_RD_HIT0__HIGH_DELTA_RESET_VAL 0
#define L3C_PROF_RD_HIT0__HIGH_DELTA_RMASK 0x1f
#define L3C_PROF_RD_HIT0__HIGH_DELTA_MASK  0x3e0

#define L3C_PROF_RD_HIT0__NEW_STATE_SHIFT 12
#define L3C_PROF_RD_HIT0__NEW_STATE_WIDTH 1
#define L3C_PROF_RD_HIT0__NEW_STATE_RESET_VAL 0
#define L3C_PROF_RD_HIT0__NEW_STATE_RMASK 0x1
#define L3C_PROF_RD_HIT0__NEW_STATE_MASK  0x1000

#define L3C_PROF_RD_HIT0__PROB_SHIFT 16
#define L3C_PROF_RD_HIT0__PROB_WIDTH 16
#define L3C_PROF_RD_HIT0__PROB_RESET_VAL 0
#define L3C_PROF_RD_HIT0__PROB_RMASK 0xffff
#define L3C_PROF_RD_HIT0__PROB_MASK  0xffff0000

#ifndef __ASSEMBLER__
__extension__
typedef union {
	struct {
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a read transactions gets a hit with that
		 * entry, in case the State Bit of that entry was 0, and the
		 * binary randomization for the transaction has resulted in
		 * 0. The index resulting from adding the delta will saturate
		 * from both directions, either at 0 (if the delta is
		 * negative) or at 15 (is the delta is positive).
		 */
		u32 low_delta    : 5;
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a read transactions gets a hit with that
		 * entry, in case the State Bit of that entry was 0, and the
		 * binary randomization for the transaction has resulted in
		 * 1. The index resulting from adding the delta will saturate
		 * from both directions, either at 0 (if the delta is
		 * negative) or at 15 (is the delta is positive).
		 */
		u32 high_delta   : 5;
		/* Reserved. */
		u32 __reserved_0 : 2;
		/*
		 * Specifies the new state of the cache entry's State Bit,
		 * after a read transaction hits it, while the current State
		 * Bit is 0.
		 */
		u32 new_state    : 1;
		/* Reserved. */
		u32 __reserved_1 : 3;
		/*
		 * The probability of a read transaction that get a hit in
		 * the cache with an entry who's State Bit is 0, to apply the
		 * Read Hit0 High Delta on the entry that it hits, rather
		 * than to the Read Hit0 Low Delta.
		 */
		u32 prob         : 16;
	};

	u32 word;
} L3C_PROF_RD_HIT0_t;
#endif /* !defined(__ASSEMBLER__) */

#define L3C_PROF_WR_HIT0__FIRST_WORD 0x06c0
#define L3C_PROF_WR_HIT0__LAST_WORD 0x06fc
#define L3C_PROF_WR_HIT0__LENGTH 0x0040
#define L3C_PROF_WR_HIT0__STRIDE 0x0004

#define L3C_PROF_WR_HIT0__LOW_DELTA_SHIFT 0
#define L3C_PROF_WR_HIT0__LOW_DELTA_WIDTH 5
#define L3C_PROF_WR_HIT0__LOW_DELTA_RESET_VAL 11
#define L3C_PROF_WR_HIT0__LOW_DELTA_RMASK 0x1f
#define L3C_PROF_WR_HIT0__LOW_DELTA_MASK  0x1f

#define L3C_PROF_WR_HIT0__HIGH_DELTA_SHIFT 5
#define L3C_PROF_WR_HIT0__HIGH_DELTA_WIDTH 5
#define L3C_PROF_WR_HIT0__HIGH_DELTA_RESET_VAL 0
#define L3C_PROF_WR_HIT0__HIGH_DELTA_RMASK 0x1f
#define L3C_PROF_WR_HIT0__HIGH_DELTA_MASK  0x3e0

#define L3C_PROF_WR_HIT0__NEW_STATE_SHIFT 12
#define L3C_PROF_WR_HIT0__NEW_STATE_WIDTH 1
#define L3C_PROF_WR_HIT0__NEW_STATE_RESET_VAL 0
#define L3C_PROF_WR_HIT0__NEW_STATE_RMASK 0x1
#define L3C_PROF_WR_HIT0__NEW_STATE_MASK  0x1000

#define L3C_PROF_WR_HIT0__PROB_SHIFT 16
#define L3C_PROF_WR_HIT0__PROB_WIDTH 16
#define L3C_PROF_WR_HIT0__PROB_RESET_VAL 0
#define L3C_PROF_WR_HIT0__PROB_RMASK 0xffff
#define L3C_PROF_WR_HIT0__PROB_MASK  0xffff0000

#ifndef __ASSEMBLER__
__extension__
typedef union {
	struct {
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a write transactions gets a hit with
		 * that entry, in case the State Bit of that entry was 0, and
		 * the binary randomization for the transaction has resulted
		 * in 0. The index resulting from adding the delta will
		 * saturate from both directions, either at 0 (if the delta
		 * is negative) or at 15 (is the delta is positive).
		 */
		u32 low_delta    : 5;
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a write transactions gets a hit with
		 * that entry, in case the State Bit of that entry was 0, and
		 * the binary randomization for the transaction has resulted
		 * in 1. The index resulting from adding the delta will
		 * saturate from both directions, either at 0 (if the delta
		 * is negative) or at 15 (is the delta is positive).
		 */
		u32 high_delta   : 5;
		/* Reserved. */
		u32 __reserved_0 : 2;
		/*
		 * Specifies the new state of the cache entry's State Bit,
		 * after a write transaction hits it, while the current State
		 * Bit is 0.
		 */
		u32 new_state    : 1;
		/* Reserved. */
		u32 __reserved_1 : 3;
		/*
		 * The probability of a write transaction that get a hit in
		 * the cache with an entry who's State Bit is 0, to apply the
		 * Write Hit0 High Delta on the entry that it hits, rather
		 * than to the Write Hit0 Low Delta.
		 */
		u32 prob         : 16;
	};

	u32 word;
} L3C_PROF_WR_HIT0_t;
#endif /* !defined(__ASSEMBLER__) */

#define L3C_PROF_RD_HIT1__FIRST_WORD 0x0700
#define L3C_PROF_RD_HIT1__LAST_WORD 0x073c
#define L3C_PROF_RD_HIT1__LENGTH 0x0040
#define L3C_PROF_RD_HIT1__STRIDE 0x0004

#define L3C_PROF_RD_HIT1__LOW_DELTA_SHIFT 0
#define L3C_PROF_RD_HIT1__LOW_DELTA_WIDTH 5
#define L3C_PROF_RD_HIT1__LOW_DELTA_RESET_VAL 11
#define L3C_PROF_RD_HIT1__LOW_DELTA_RMASK 0x1f
#define L3C_PROF_RD_HIT1__LOW_DELTA_MASK  0x1f

#define L3C_PROF_RD_HIT1__HIGH_DELTA_SHIFT 5
#define L3C_PROF_RD_HIT1__HIGH_DELTA_WIDTH 5
#define L3C_PROF_RD_HIT1__HIGH_DELTA_RESET_VAL 0
#define L3C_PROF_RD_HIT1__HIGH_DELTA_RMASK 0x1f
#define L3C_PROF_RD_HIT1__HIGH_DELTA_MASK  0x3e0

#define L3C_PROF_RD_HIT1__NEW_STATE_SHIFT 12
#define L3C_PROF_RD_HIT1__NEW_STATE_WIDTH 1
#define L3C_PROF_RD_HIT1__NEW_STATE_RESET_VAL 0
#define L3C_PROF_RD_HIT1__NEW_STATE_RMASK 0x1
#define L3C_PROF_RD_HIT1__NEW_STATE_MASK  0x1000

#define L3C_PROF_RD_HIT1__PROB_SHIFT 16
#define L3C_PROF_RD_HIT1__PROB_WIDTH 16
#define L3C_PROF_RD_HIT1__PROB_RESET_VAL 0
#define L3C_PROF_RD_HIT1__PROB_RMASK 0xffff
#define L3C_PROF_RD_HIT1__PROB_MASK  0xffff0000

#ifndef __ASSEMBLER__
__extension__
typedef union {
	struct {
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a read transactions gets a hit with that
		 * entry, in case the State Bit of that entry was 1, and the
		 * binary randomization for the transaction has resulted in
		 * 0. The index resulting from adding the delta will saturate
		 * from both directions, either at 0 (if the delta is
		 * negative) or at 15 (is the delta is positive).
		 */
		u32 low_delta    : 5;
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a read transactions gets a hit with that
		 * entry, in case the State Bit of that entry was 1, and the
		 * binary randomization for the transaction has resulted in
		 * 1. The index resulting from adding the delta will saturate
		 * from both directions, either at 0 (if the delta is
		 * negative) or at 15 (is the delta is positive).
		 */
		u32 high_delta   : 5;
		/* Reserved. */
		u32 __reserved_0 : 2;
		/*
		 * Specifies the new state of the cache entry's State Bit,
		 * after a read transaction hits it, while the current State
		 * Bit is 1.
		 */
		u32 new_state    : 1;
		/* Reserved. */
		u32 __reserved_1 : 3;
		/*
		 * The probability of a read transaction that get a hit in
		 * the cache with an entry who's State Bit is 1, to apply the
		 * Read Hit1 High Delta on the entry that it hits, rather
		 * than to the Read Hit1 Low Delta.
		 */
		u32 prob         : 16;
	};

	u32 word;
} L3C_PROF_RD_HIT1_t;
#endif /* !defined(__ASSEMBLER__) */

#define L3C_PROF_WR_HIT1__FIRST_WORD 0x0740
#define L3C_PROF_WR_HIT1__LAST_WORD 0x077c
#define L3C_PROF_WR_HIT1__LENGTH 0x0040
#define L3C_PROF_WR_HIT1__STRIDE 0x0004

#define L3C_PROF_WR_HIT1__LOW_DELTA_SHIFT 0
#define L3C_PROF_WR_HIT1__LOW_DELTA_WIDTH 5
#define L3C_PROF_WR_HIT1__LOW_DELTA_RESET_VAL 11
#define L3C_PROF_WR_HIT1__LOW_DELTA_RMASK 0x1f
#define L3C_PROF_WR_HIT1__LOW_DELTA_MASK  0x1f

#define L3C_PROF_WR_HIT1__HIGH_DELTA_SHIFT 5
#define L3C_PROF_WR_HIT1__HIGH_DELTA_WIDTH 5
#define L3C_PROF_WR_HIT1__HIGH_DELTA_RESET_VAL 0
#define L3C_PROF_WR_HIT1__HIGH_DELTA_RMASK 0x1f
#define L3C_PROF_WR_HIT1__HIGH_DELTA_MASK  0x3e0

#define L3C_PROF_WR_HIT1__NEW_STATE_SHIFT 12
#define L3C_PROF_WR_HIT1__NEW_STATE_WIDTH 1
#define L3C_PROF_WR_HIT1__NEW_STATE_RESET_VAL 0
#define L3C_PROF_WR_HIT1__NEW_STATE_RMASK 0x1
#define L3C_PROF_WR_HIT1__NEW_STATE_MASK  0x1000

#define L3C_PROF_WR_HIT1__PROB_SHIFT 16
#define L3C_PROF_WR_HIT1__PROB_WIDTH 16
#define L3C_PROF_WR_HIT1__PROB_RESET_VAL 0
#define L3C_PROF_WR_HIT1__PROB_RMASK 0xffff
#define L3C_PROF_WR_HIT1__PROB_MASK  0xffff0000

#ifndef __ASSEMBLER__
__extension__
typedef union {
	struct {
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a write transactions gets a hit with
		 * that entry, in case the State Bit of that entry was 1, and
		 * the binary randomization for the transaction has resulted
		 * in 0. The index resulting from adding the delta will
		 * saturate from both directions, either at 0 (if the delta
		 * is negative) or at 15 (is the delta is positive).
		 */
		u32 low_delta    : 5;
		/*
		 * A signed addition to LRU structure index of a cache entry,
		 * that is used when a write transactions gets a hit with
		 * that entry, in case the State Bit of that entry was 1, and
		 * the binary randomization for the transaction has resulted
		 * in 1. The index resulting from adding the delta will
		 * saturate from both directions, either at 0 (if the delta
		 * is negative) or at 15 (is the delta is positive).
		 */
		u32 high_delta   : 5;
		/* Reserved. */
		u32 __reserved_0 : 2;
		/*
		 * Specifies the new state of the cache entry's State Bit,
		 * after a write transaction hits it, while the current State
		 * Bit is 1.
		 */
		u32 new_state    : 1;
		/* Reserved. */
		u32 __reserved_1 : 3;
		/*
		 * The probability of a write transaction that get a hit in
		 * the cache with an entry who's State Bit is 1, to apply the
		 * Write Hit1 High Delta on the entry that it hits, rather
		 * than to the Write Hit1 Low Delta.
		 */
		u32 prob         : 16;
	};

	u32 word;
} L3C_PROF_WR_HIT1_t;
#endif /* !defined(__ASSEMBLER__) */

#endif /* !defined(__L3CACHE_REGS_H__) */
